@@ -21,9 +21,10 @@ import (
 	"errors"
 	"fmt"
 
+	"github.com/digitalocean/godo"
 	apiv1 "k8s.io/api/core/v1"
+
 	"k8s.io/autoscaler/cluster-autoscaler/cloudprovider"
-	"k8s.io/autoscaler/cluster-autoscaler/cloudprovider/digitalocean/godo"
 	schedulernodeinfo "k8s.io/kubernetes/pkg/scheduler/nodeinfo"
 )
 
