@@ -19,9 +19,10 @@ package test
 import (
 	"time"
 
+	autoscaling "k8s.io/api/autoscaling/v1"
 	core "k8s.io/api/core/v1"
 	meta "k8s.io/apimachinery/pkg/apis/meta/v1"
-	vpa_types "k8s.io/autoscaler/vertical-pod-autoscaler/pkg/apis/autoscaling.k8s.io/v1beta2"
+	vpa_types "k8s.io/autoscaler/vertical-pod-autoscaler/pkg/apis/autoscaling.k8s.io/v1"
 )
 
 // VerticalPodAutoscalerBuilder helps building test instances of VerticalPodAutoscaler.
@@ -35,31 +36,36 @@ type VerticalPodAutoscalerBuilder interface {
 	WithMaxAllowed(cpu, memory string) VerticalPodAutoscalerBuilder
 	WithTarget(cpu, memory string) VerticalPodAutoscalerBuilder
 	WithLowerBound(cpu, memory string) VerticalPodAutoscalerBuilder
+	WithTargetRef(targetRef *autoscaling.CrossVersionObjectReference) VerticalPodAutoscalerBuilder
 	WithUpperBound(cpu, memory string) VerticalPodAutoscalerBuilder
 	AppendCondition(conditionType vpa_types.VerticalPodAutoscalerConditionType,
 		status core.ConditionStatus, reason, message string, lastTransitionTime time.Time) VerticalPodAutoscalerBuilder
+	AppendRecommendation(vpa_types.RecommendedContainerResources) VerticalPodAutoscalerBuilder
 	Get() *vpa_types.VerticalPodAutoscaler
 }
 
 // VerticalPodAutoscaler returns a new VerticalPodAutoscalerBuilder.
 func VerticalPodAutoscaler() VerticalPodAutoscalerBuilder {
 	return &verticalPodAutoscalerBuilder{
-		recommendation: Recommendation(),
-		namespace:      "default",
-		conditions:     []vpa_types.VerticalPodAutoscalerCondition{},
+		recommendation:          Recommendation(),
+		appendedRecommendations: []vpa_types.RecommendedContainerResources{},
+		namespace:               "default",
+		conditions:              []vpa_types.VerticalPodAutoscalerCondition{},
 	}
 }
 
 type verticalPodAutoscalerBuilder struct {
-	vpaName           string
-	containerName     string
-	namespace         string
-	updatePolicy      *vpa_types.PodUpdatePolicy
-	creationTimestamp time.Time
-	minAllowed        core.ResourceList
-	maxAllowed        core.ResourceList
-	recommendation    RecommendationBuilder
-	conditions        []vpa_types.VerticalPodAutoscalerCondition
+	vpaName                 string
+	containerName           string
+	namespace               string
+	updatePolicy            *vpa_types.PodUpdatePolicy
+	creationTimestamp       time.Time
+	minAllowed              core.ResourceList
+	maxAllowed              core.ResourceList
+	recommendation          RecommendationBuilder
+	conditions              []vpa_types.VerticalPodAutoscalerCondition
+	targetRef               *autoscaling.CrossVersionObjectReference
+	appendedRecommendations []vpa_types.RecommendedContainerResources
 }
 
 func (b *verticalPodAutoscalerBuilder) WithName(vpaName string) VerticalPodAutoscalerBuilder {
@@ -125,6 +131,12 @@ func (b *verticalPodAutoscalerBuilder) WithUpperBound(cpu, memory string) Vertic
 	return &c
 }
 
+func (b *verticalPodAutoscalerBuilder) WithTargetRef(targetRef *autoscaling.CrossVersionObjectReference) VerticalPodAutoscalerBuilder {
+	c := *b
+	c.targetRef = targetRef
+	return &c
+}
+
 func (b *verticalPodAutoscalerBuilder) AppendCondition(conditionType vpa_types.VerticalPodAutoscalerConditionType,
 	status core.ConditionStatus, reason, message string, lastTransitionTime time.Time) VerticalPodAutoscalerBuilder {
 	c := *b
@@ -137,6 +149,12 @@ func (b *verticalPodAutoscalerBuilder) AppendCondition(conditionType vpa_types.V
 	return &c
 }
 
+func (b *verticalPodAutoscalerBuilder) AppendRecommendation(recommendation vpa_types.RecommendedContainerResources) VerticalPodAutoscalerBuilder {
+	c := *b
+	c.appendedRecommendations = append(c.appendedRecommendations, recommendation)
+	return &c
+}
+
 func (b *verticalPodAutoscalerBuilder) Get() *vpa_types.VerticalPodAutoscaler {
 	if b.containerName == "" {
 		panic("Must call WithContainer() before Get()")
@@ -147,6 +165,11 @@ func (b *verticalPodAutoscalerBuilder) Get() *vpa_types.VerticalPodAutoscaler {
 		MaxAllowed:    b.maxAllowed,
 	}}}
 
+	recommendation := b.recommendation.WithContainer(b.containerName).Get()
+	for _, rec := range b.appendedRecommendations {
+		recommendation.ContainerRecommendations = append(recommendation.ContainerRecommendations, rec)
+	}
+
 	return &vpa_types.VerticalPodAutoscaler{
 		ObjectMeta: meta.ObjectMeta{
 			Name:              b.vpaName,
@@ -156,9 +179,10 @@ func (b *verticalPodAutoscalerBuilder) Get() *vpa_types.VerticalPodAutoscaler {
 		Spec: vpa_types.VerticalPodAutoscalerSpec{
 			UpdatePolicy:   b.updatePolicy,
 			ResourcePolicy: &resourcePolicy,
+			TargetRef:      b.targetRef,
 		},
 		Status: vpa_types.VerticalPodAutoscalerStatus{
-			Recommendation: b.recommendation.WithContainer(b.containerName).Get(),
+			Recommendation: recommendation,
 			Conditions:     b.conditions,
 		},
 	}
